@@ -3,7 +3,11 @@ package PDF::Imposition::Schema1x4x2cutfoldbind;
 use strict;
 use warnings;
 
-use base qw/PDF::Imposition::Schema/;
+use Types::Standard qw/Bool/;
+use namespace::clean;
+
+use Moo;
+with 'PDF::Imposition::Schema';
 
 =head1 NAME
 
@@ -42,8 +46,48 @@ B<bind> the spine. (Hence the name of the module).
 Does it sound weird? Well, kind of. Looks like a lot of manual work.
 But if it works for you, it works for me as well.
 
+=head1 METHODS
+
+=head2 cover
+
+This schema supports the cover option.
+
 =cut
 
+has cover => (is => 'rw', isa => Bool);
+
+sub pages_per_sheet { 8 }
+
+sub signature { 8 }
+
+=head1 INTERNALS
+
+=head2 signature
+
+Returns 8.
+
+=head2 pages_per_sheet
+
+Returns 8.
+
+=head2 cropmarks_options
+
+Set inner to false and force the signature to 4 for cropmarks purposes.
+
+=cut
+
+sub cropmarks_options {
+    my %options = (
+                   top => 1,
+                   bottom => 1,
+                   inner => 0,
+                   outer => 1,
+                   twoside => 1,
+                   # for the purpose of the cropping, the signature is 4
+                   signature => 4,
+                  );
+    return %options;
+}
 
 sub _do_impose {
     my $self = shift;
@@ -69,8 +113,6 @@ sub _do_impose {
         $self->_compose_quadruple($p8, $p5, $p1, $p4);
         $self->_compose_quadruple($p6, $p7, $p3, $p2);
     }
-    $self->out_pdf_obj->saveas($self->outfile);
-    return $self->outfile;
 }
 
 sub _compose_quadruple {
